@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -26,8 +26,9 @@ failing examples it finds.
 
 from hypothesis._settings import settings, Verbosity, Phase, HealthCheck
 from hypothesis.version import __version_info__, __version__
-from hypothesis.control import assume, note, reject
+from hypothesis.control import assume, note, reject, event
 from hypothesis.core import given, find, example, seed
+from hypothesis.utils.conventions import infer
 
 
 __all__ = [
@@ -42,6 +43,8 @@ __all__ = [
     'find',
     'example',
     'note',
+    'event',
+    'infer',
     '__version__',
     '__version_info__',
 ]
