@@ -26,6 +26,7 @@ import traceback
 from random import Random
 from collections import namedtuple
 
+import hypothesis.strategies as st
 from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     Unsatisfiable, InvalidArgument, FailedHealthCheck, \
     UnsatisfiedAssumption, HypothesisDeprecationWarning
@@ -36,7 +37,9 @@ from hypothesis.executors import new_style_executor, \
     default_new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
-from hypothesis.internal.compat import str_to_bytes, getfullargspec
+from hypothesis.internal.compat import str_to_bytes, get_type_hints, \
+    getfullargspec
+from hypothesis.utils.conventions import infer
 from hypothesis.internal.escalation import \
     escalate_hypothesis_internal_error
 from hypothesis.internal.reflection import nicerepr, arg_string, \
@@ -153,22 +156,25 @@ def is_invalid_test(
         return invalid(
             'given must be called with at least one argument')
 
-    if (generator_arguments and (original_argspec.varargs or
-                                 original_argspec.varkw)):
+    if generator_arguments and any([original_argspec.varargs,
+                                    original_argspec.varkw,
+                                    original_argspec.kwonlyargs]):
         return invalid(
-            'varargs or keywords are not supported with positional '
-            'arguments to @given'
+            'positional arguments to @given are not supported with varargs, '
+            'varkeywords, or keyword-only arguments'
         )
 
-    if (
-        len(generator_arguments) > len(original_argspec.args)
-    ):
+    if len(generator_arguments) > len(original_argspec.args):
         return invalid((
             'Too many positional arguments for %s() (got %d but'
             ' expected at most %d') % (
                 name, len(generator_arguments),
                 len(original_argspec.args)))
 
+    if infer in generator_arguments:
+        return invalid('infer was passed as a positional argument to @given, '
+                       'but may only be passed as a keyword argument')
+
     if generator_arguments and generator_kwargs:
         return invalid(
             'cannot mix positional and keyword arguments to @given'
@@ -405,8 +411,6 @@ def process_arguments_to_given(
     wrapped_test, arguments, kwargs, generator_arguments, generator_kwargs,
     argspec, test, settings
 ):
-    import hypothesis.strategies as sd
-
     selfy = None
     arguments, kwargs = convert_positional_arguments(
         wrapped_test, arguments, kwargs)
@@ -422,14 +426,13 @@ def process_arguments_to_given(
 
     arguments = tuple(arguments)
 
-    given_specifier = sd.tuples(
-        sd.just(arguments),
-        sd.fixed_dictionaries(generator_kwargs).map(
+    search_strategy = st.tuples(
+        st.just(arguments),
+        st.fixed_dictionaries(generator_kwargs).map(
             lambda args: dict(args, **kwargs)
         )
     )
 
-    search_strategy = given_specifier
     if selfy is not None:
         search_strategy = WithRunner(search_strategy, selfy)
 
@@ -473,6 +476,18 @@ def skip_exceptions_to_reraise():
 exceptions_to_reraise = skip_exceptions_to_reraise()
 
 
+def new_given_argspec(original_argspec, generator_kwargs):
+    """Make an updated argspec for the wrapped test."""
+    new_args = [a for a in original_argspec.args if a not in generator_kwargs]
+    new_kwonlyargs = [a for a in original_argspec.kwonlyargs
+                      if a not in generator_kwargs]
+    annots = {k: v for k, v in original_argspec.annotations.items()
+              if k in new_args + new_kwonlyargs}
+    annots['return'] = None
+    return original_argspec._replace(
+        args=new_args, kwonlyargs=new_kwonlyargs, annotations=annots)
+
+
 class StateForActualGivenExecution(object):
 
     def __init__(self, test_runner, search_strategy, test, settings, random):
@@ -629,20 +644,11 @@ def given(*given_arguments, **given_kwargs):
         if check_invalid is not None:
             return check_invalid
 
-        arguments = original_argspec.args
-
-        for name, strategy in zip(arguments[-len(generator_arguments):],
-                                  generator_arguments):
+        for name, strategy in zip(reversed(original_argspec.args),
+                                  reversed(generator_arguments)):
             generator_kwargs[name] = strategy
 
-        argspec = original_argspec._replace(
-            args=[a for a in arguments if a not in generator_kwargs],
-            defaults=None,
-        )
-        annots = {k: v for k, v in argspec.annotations.items()
-                  if k in (argspec.args + argspec.kwonlyargs)}
-        annots['return'] = None
-        argspec = argspec._replace(annotations=annots)
+        argspec = new_given_argspec(original_argspec, generator_kwargs)
 
         @impersonate(test)
         @define_function_signature(
@@ -656,6 +662,16 @@ def given(*given_arguments, **given_kwargs):
 
             random = get_random_for_wrapped_test(test, wrapped_test)
 
+            if infer in generator_kwargs.values():
+                hints = get_type_hints(test)
+            for name in [name for name, value in generator_kwargs.items()
+                         if value is infer]:
+                if name not in hints:
+                    raise InvalidArgument(
+                        'passed %s=infer for %s, but %s has no type annotation'
+                        % (name, test.__name__, name))
+                generator_kwargs[name] = st.from_type(hints[name])
+
             processed_args = process_arguments_to_given(
                 wrapped_test, arguments, kwargs, generator_arguments,
                 generator_kwargs, argspec, test, settings
